--[[----------------------------------------------------------------------------
	Copyright 2007-2009 João Afonso Libório Cardoso
	All rights reserved
	
	MailNotifier is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3 as published by the Free Software Foundation.
	MailNotifier is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License (<http://www.gnu.org/licenses/>) for more details.
---------------------------------------------------------------------------]]--

local MailNotifier = CreateFrame('Frame', 'MailNotifier')
local Indicator, Shine, WrongEvent, NewMails, AtInbox, sets
local L = MailNotifier_Locals


--[[ Startup ]]--

function MailNotifier:Load()
	Indicator = MiniMapMailFrame:CreateFontString('MailNotifierIndicator', 'OVERLAY')
	Indicator:SetFontObject('NumberFontNormal')
	Indicator:SetPoint('Center', MiniMapMailFrame, 'Center', -1, 1)
	Indicator:SetJustifyH('CENTER')

	Shine = MiniMapMailFrame:CreateTexture('MailNotifierShine', 'OVERLAY')
	Shine:SetTexture('Interface\\Calendar\\EventNotificationGlow')
	Shine:SetPoint('CENTER', MiniMapMailFrame, 'CENTER', -2, 2)
	Shine:SetHeight(35) Shine:SetWidth(35)
	Shine:SetBlendMode('ADD')
	Shine:Hide()
	
	self:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
	self:RegisterEvent('UPDATE_PENDING_MAIL')
	self:RegisterEvent('VARIABLES_LOADED')
	self:RegisterEvent('PLAYER_LOGOUT')
	self:RegisterEvent('PLAYER_ALIVE')
	self:RegisterEvent('PLAYER_DEAD')
	self:RegisterEvent('MAIL_CLOSED')
	self:RegisterEvent('MAIL_SHOW')
end


--[[ SavedVariables ]]--

function MailNotifier:VARIABLES_LOADED()
	MailNotifier_SV = MailNotifier_SV or {unread = 0, senders = {}}
	sets = MailNotifier_SV
end

function MailNotifier:PLAYER_LOGOUT()
	wipe(sets.senders)
	for i = 1, select('#', GetLatestThreeSenders()) do
		sets.senders[select(i, GetLatestThreeSenders())] = i
	end
end


--[[ Mail API ]]--

function MailNotifier:AddNewMail(new, silent)
	self:SetNewMails(sets.unread + (new or 1))
	if not silent then
		self:AlertMail()
	end
end

function MailNotifier:AlertMail()
	PlaySoundFile("Interface\\AddOns\\MailNotifier\\NewMail.wav")
	UIFrameFlash(Shine, 1, 1, 10, nil, 0.6)
end

function MailNotifier:SetNewMails(value)
	Indicator:SetText(value or '')
	sets.unread = value
end

function MailNotifier:GetNewMails()
	return sets.unread
end


--[[ Outdoor ]]--

function MailNotifier:PLAYER_ALIVE()
	WrongEvent = true
end

function MailNotifier:PLAYER_DEAD()
	WrongEvent = true
end

function MailNotifier:PLAYER_ENTERING_WORLD()
	WrongEvent = true
end

function MailNotifier:UPDATE_PENDING_MAIL()
	if HasNewMail() then
		local NewSenders = {GetLatestThreeSenders()}
		local OldSenders = sets.senders
		local newMails, oldI, off = 0
		
		for newI, sender in pairs(NewSenders) do
			oldI = OldSenders[sender] or 0
				
			if newI >= oldI then
				off = newI - oldI
			else
				off = 3 - oldI + newI
			end
				
			newMails = max(newMails, off)
		end
		
		self:AddNewMail(newMails, newMails == 0)
	else
		WrongEvent = nil
	end
	
	self:UnregisterEvent('PLAYER_DEAD')
	self:UnregisterEvent('PLAYER_ALIVE')
	self:RegisterEvent('CHAT_MSG_SYSTEM')
	self:RegisterEvent('PLAYER_ENTERING_WORLD')

	function MailNotifier:UPDATE_PENDING_MAIL() --has a "startup" function and then changes to the basic one
		if WrongEvent then
			WrongEvent = nil
		elseif not AtInbox then
			self:AddNewMail()
		end
	end
end


--[[ Inbox ]]--

function MailNotifier:MAIL_SHOW()
	self:RegisterEvent('MAIL_INBOX_UPDATE')
	self:UnregisterEvent('MAIL_SHOW')
end

function MailNotifier:MAIL_INBOX_UPDATE()
	self:SetNewMails(self:GetInboxNewMails())
	
	if not AtInbox then
		NewMails = self:GetNewMails()
		AtInbox = true
	else
		if self:GetNewMails() ~= NewMails then
			WrongEvent = true
		end
	end
end

function MailNotifier:MAIL_CLOSED()
	AtInbox = nil
end

function MailNotifier:GetInboxNewMails()
	local newMails = 0
	for i = 1, GetInboxNumItems() do
		if not select(9, GetInboxHeaderInfo(i)) then
			newMails = newMails + 1
		end
	end
	return newMails
end


--[[ System Messages ]]--

function MailNotifier:CHAT_MSG_SYSTEM(message)
	if self:MatchSystemMessage(message, ERR_AUCTION_OUTBID_S) then
		MiniMapMailFrame:GetScript('OnEvent')()
		self:AddNewMail()
	elseif arg1 == ERR_AUCTION_REMOVED or self:MatchSystemMessage(message, ERR_AUCTION_WON_S) then
		WrongEvent = true
	end
end

function MailNotifier:MatchSystemMessage(arg1, arg2)
	arg2 = strsplit("%%s", arg2)
	return strfind(arg1, arg2)
end


--[[ Tootlip ]]--

function MinimapMailFrameUpdate()
	local senders = {GetLatestThreeSenders()}
	local title
	
	if MailNotifier:GetNewMails() ~= 1 then
		title = L.HaveMails
	else
		title = L.HaveMail
	end
			
	if #senders > 0 then
		title = title .. L.From
	end
	
	GameTooltip:SetText(format(title, sets.unread))
	
	for i,sender in pairs(senders) do
		GameTooltip:AddLine(' - ' .. sender, 1, 1, 1)
	end
	
	GameTooltip:Show()
end


--[[ Call Addon ]]--

MailNotifier:Load()